package cc.fascinated.common;

import lombok.experimental.UtilityClass;

import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.Locale;

/**
 * @author Fascinated (fascinated7)
 */
@UtilityClass
public class DateUtils {
    private static final ZoneId ZONE_ID = ZoneId.of("Europe/London");
    private static final DateTimeFormatter ISO_FORMATTER = DateTimeFormatter.ISO_INSTANT
            .withLocale(Locale.UK)
            .withZone(ZONE_ID);
    private static final DateTimeFormatter SIMPLE_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd")
            .withLocale(Locale.UK)
            .withZone(ZONE_ID);

    /**
     * Gets the date from a string.
     *
     * @param date The date string.
     * @return The date.
     */
    public static Date getDateFromIsoString(String date) {
        return Date.from(Instant.from(ISO_FORMATTER.parse(date)));
    }

    /**
     * Gets the date from a string.
     *
     * @param date The date string.
     * @return The date.
     */
    public static Date getDateFromString(String date) {
        LocalDate localDate = LocalDate.parse(date, SIMPLE_FORMATTER);
        ZonedDateTime zonedDateTime = localDate.atStartOfDay(ZONE_ID);
        return Date.from(zonedDateTime.toInstant());
    }

    /**
     * Formats a date to a string.
     *
     * @param date The date to format.
     * @return The formatted date.
     */
    public String formatDate(Date date) {
        return SIMPLE_FORMATTER.format(date.toInstant());
    }

    /**
     * Aligns the date to the current hour.
     * <p>
     * eg: 00:05 -> 00:00
     * </p>
     *
     * @param date The date to align.
     * @return The aligned date.
     */
    public static Date alignToCurrentHour(Date date) {
        return Date.from(Instant.ofEpochMilli(date.getTime()).truncatedTo(ChronoUnit.HOURS));
    }

    /**
     * Gets the date from an amount of days ago.
     *
     * @param days The amount to go back.
     * @return The date.
     */
    public static Date getDaysAgo(int days) {
        return Date.from(Instant.now().minus(days, ChronoUnit.DAYS));
    }

    /**
     * Gets the date for midnight today.
     *
     * @return The date.
     */
    public static Date getMidnightToday() {
        return Date.from(Instant.now().truncatedTo(ChronoUnit.DAYS));
    }
}
